;-------------------------------------------------------------------------------
;	Periferie DS1 - zaplava
;	DS adresa 00000100
;
;	16F84A / 4MHz			
;	vse OFF, osc. XT, bez preruseni
;
;	06/2016 Milan rajer
;-------------------------------------------------------------------------------
;
;                             --------\/--------
;                            -|RA2          RA1|-  BUS
;                       Z1a  -|RA3          RA0|-  VODA
;                       Z1b  -|RA4         OSC1|-  \ Krystal 4,00 Mhz
;                            -|MCLR        OSC2|-  /
;                        0V  -|Vss          Vdd|-  +5V
;                       Z2a  -|RB0          RB7|-  ICSP
;                       Z2b  -|RB1          RB6|-  ICSP
;                       Z3a  -|RB2          RB5|-  Z4a
;                       Z3b  -|RB3          RB4|-  Z4b
;                             ------------------
;
;-------------------------------------------------------------------------------

	LIST P=16F84, R=DEC
	INCLUDE <P16F84.INC>

;-------------------------------
;	Nastaveni pojistek
;-------------------------------

	org	0x2007		; adresa nastaveni pojistek pro PIC16F84
	dw _XT_OSC & _PWRTE_OFF & _WDT_OFF & _CP_OFF

;-------------------------------
;	Nastaveni dat v EEPROM
;-------------------------------

	org	0x2100		;tabulka v pameti EEPROM (64 bytu)
;		----------------------------------------------
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
;		----------------------------------------------

;--------------------------------------------------------------------------------
;	Nastaveni parametru
;-------------------------------
W	equ	0
F	equ	1
;--------------------------------------------------------------------------------
;	Definice registru, bitu a pinu
;-------------------------------
SYS	equ	0x0C
;-----------------------------------
;|   |   |   |   |   |   |   | TMP |
;-----------------------------------
#define	TMP	SYS,0
;       ------------------------
TMP1	equ	0x0D		;pomocny registr
TMP2	equ	0x0E		;pomocny registr
TMP3	equ	0x0F		;pomocny registr
TMP4	equ	0x10		;pomocny registr
TMP5	equ	0x11		;pomocny registr
;       ------------------------
CEK1	equ	0x12		;casova smycka
CEK2	equ	0x13		;casova smycka
CEK3	equ	0x14		;casova smycka
;       ------------------------
DAT	equ	0x15		;datovy registr
;
ZAPLAVA	equ	0x16		;registr zaplavovych cidel
ZAPLAV2	equ	0x17		;registr dvojic zaplavovych cidel
;-------------------------------
;	Sbernice
;-------------------------------
#define	BUS	PORTA,1		;sbernice
#define	BUS_T	TRISA,1		;tris registr sbernice
;-------------------------------
;	Periferie
;-------------------------------
#define	Z1A	PORTA,3		;zaplava 1 a
#define	Z1B	PORTA,4		;zaplava 1 b
#define	Z2A	PORTB,0		;zaplava 2 a
#define	Z2B	PORTB,1		;zaplava 2 b
#define	Z3A	PORTB,2		;zaplava 3 a
#define	Z3B	PORTB,3		;zaplava 3 b
#define	Z4A	PORTB,5		;zaplava 4 a
#define	Z4B	PORTB,4		;zaplava 4 b

#define	VODA	PORTA,0		;pripadne ovladani ventilu
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;	Zacatek behu programu
;-------------------------------
	org	0x0000
	goto	initial
;--------------------------------------------------------------------------------
;	Knihovny
;-------------------------------
	#include	<piclib_bus_4MHz.inc>		;komunikace po sbernici
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;	Inicializace po zapnuti
;-------------------------------
;-------------------------------
;	nastaveni portu
;-------------------------------
initial	bcf	STATUS,RP0	;banka 0
	bcf	STATUS,RP1
;-------------------------------
	clrf	PORTA		;vymazani vsech portu
	clrf	PORTB
;-------------------------------
	bsf	STATUS,RP0	;banka 1
	bcf	STATUS,RP1
;-------------------------------
	movlw	b'11110'
	movwf	TRISA		;port A vstupni/vystupni
	movlw	b'11111111'
	movwf	TRISB		;port B vstupni/vystupni
;       ------------------------
	bcf	STATUS,RP0	;banka 0
	bcf	STATUS,RP1
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;
;	*****	*****	*****	****	*****
;	*	  *	*   *	*   *	  *
;	*****	  *	***** 	****	  *
;	    *	  *	*   *	*  *	  *
;	*****	  *	*   *	*   *	  *
;
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
start	clrf	ZAPLAVA		;vynuluj registry
;-------------------------------
;
;	Zjisteni stavu zaplavovych cidel
;
;	1 = vybavene cidlo
;
;-------------------------------
mereni	btfss	Z1A		;Z1A
	bsf	ZAPLAVA,0
	btfsc	Z1A
	bcf	ZAPLAVA,0
;       ------------------------
	btfss	Z1B		;Z1B
	bsf	ZAPLAVA,1
	btfsc	Z1B
	bcf	ZAPLAVA,1
;-------------------------------
	btfss	Z2A		;Z2A
	bsf	ZAPLAVA,2
	btfsc	Z2A
	bcf	ZAPLAVA,2
;       ------------------------
	btfss	Z2B		;Z2B
	bsf	ZAPLAVA,3
	btfsc	Z2B
	bcf	ZAPLAVA,3
;-------------------------------
	btfss	Z3A		;Z3A
	bsf	ZAPLAVA,4
	btfsc	Z3A
	bcf	ZAPLAVA,4
;       ------------------------
	btfss	Z3B		;Z3B
	bsf	ZAPLAVA,5
	btfsc	Z3B
	bcf	ZAPLAVA,5
;-------------------------------
	btfss	Z4A		;Z4A
	bsf	ZAPLAVA,6
	btfsc	Z4A
	bcf	ZAPLAVA,6
;       ------------------------
	btfss	Z4B		;Z4B
	bsf	ZAPLAVA,7
	btfsc	Z4B
	bcf	ZAPLAVA,7
;-------------------------------
	clrf	ZAPLAV2
;-------------------------------
mZ1	btfsc	Z1A
	goto	mZ2
	btfsc	Z1B
	goto	mZ2
;       ------------------------
	bsf	ZAPLAV2,0	;jsou vybavena obe cidla Z1
;-------------------------------
mZ2	btfsc	Z2A
	goto	mZ3
	btfsc	Z2B
	goto	mZ3
;       ------------------------
	bsf	ZAPLAV2,1	;jsou vybavena obe cidla Z2
;-------------------------------
mZ3	btfsc	Z3A
	goto	mZ4
	btfsc	Z3B
	goto	mZ4
;       ------------------------
	bsf	ZAPLAV2,2	;jsou vybavena obe cidla Z3
;-------------------------------
mZ4	btfsc	Z4A
	goto	mZ
	btfsc	Z4B
	goto	mZ
;       ------------------------
	bsf	ZAPLAV2,3	;jsou vybavena obe cidla Z4
;-------------------------------
mZ	movf	ZAPLAV2,F
	btfsc	STATUS,Z	;pokud neni zaplava, nuluj ZAPLAV2,7
	bcf	ZAPLAV2,7
	btfss	STATUS,Z	;pokud je nejaka zaplava, nastav ZAPLAV2,7
	bsf	ZAPLAV2,7
;-------------------------------
;
;	Nastaveni vystupu VODA
;
;	1 = zaplava - uzavrit ventil
;
;-------------------------------
	movf	ZAPLAV2,F
	btfsc	STATUS,Z	;pokud neni zaplava, nuluj VODA
	bcf	VODA
	btfss	STATUS,Z	;pokud je zaplava, nastav VODA
	bsf	VODA
;-------------------------------
;	Neustale probihajici detekce reset impulsu
;-------------------------------
rutina	btfsc	BUS		;cekej na log 0
	goto	rutina
;-------------------------------
	movlw	.30		;test delky impulsu log 0 po dobu 120us
        movwf	CEK1
;       ------------------------
	btfsc	BUS		;pokud je impuls kratsi, vrat se na rutinu
	goto	rutina
;       ------------------------
	decfsz	CEK1,1
        goto	$-3		;..jinak pokracuj
;-------------------------------
	movlw	.3		;cekej max 1200us na konec reset impulsu (log 1)
        movwf	CEK2
rut12   movlw	.100
        movwf	CEK1
;       ------------------------
rut11	btfsc	BUS		;pokud behem testu prejde BUS do log 1..
	goto	nac_adr		;..pokracuj v prijmu
;       ------------------------
	decfsz	CEK1,1
        goto	rut11
        decfsz	CEK2,1
        goto	rut12		;pokud se log 1 neobevi, vrat se na rutinu
	goto	rutina
;-------------------------------
;	Prijem adresy periferie (00000100)
;-------------------------------
nac_adr	call	bus_dat		;precte byte ze sbernice
;       ------------------------
	movlw	b'00000100'	;je tazano toto zarizeni?
	subwf	DAT,W
	btfss	STATUS,Z	;pokud ne, vrat se na rutinu, jinak pokracuj
	goto	rutina
;-------------------------------
;	Vyslani dat na sbernici
;-------------------------------
vysli	call	bus_rw		;prodleva pred zacatkem vysilani
;       ------------------------
	movf	ZAPLAVA,W	;ZAPLAVA
	call	w_bus
;       ------------------------
	movf	ZAPLAV2,W	;ZAPLAV2
	call	w_bus
;       ------------------------
	movlw	b'01010101'	;kontrolni byte
	call	w_bus
;       ------------------------
	goto	mereni		;po vyslani dat se vrat a proved nove mereni
;--------------------------------------------------------------------------------
	end